'use strict';

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _utils = require('./utils');

/**
 * A utility class to manage a single defferable task.
 *
 * @class Task
 */

var Task = (function () {
  function Task() {
    _classCallCheck(this, Task);
  }

  _createClass(Task, [{
    key: 'isSet',

    /**
     * Checks if a task is set to execute in the future.
     *
     * @returns {boolean} True if a task is set.
     */

    value: function isSet() {
      return !(0, _utils.isUndefined)(this._id);
    }

    /**
     * Defers a function to be executed on the next turn of the event loop.
     * Will not allow a defer if the task is already set to execute.
     *
     * @param {Function} Callback to execute.
     */

  }, {
    key: 'defer',
    value: function defer(callback) {
      var _this = this;

      (0, _utils.invariant)(!this.isSet(), 'Cannot set a new task while another is currently set.');

      this._id = (0, _utils.defer)(function () {
        _this.clear();
        callback();
      });
    }

    /**
     * Clears the task this class is managing. If the task has not run,
     * it will never run. If the task has run, the class will be opened
     * up for a new callback to be deffered.
     */

  }, {
    key: 'clear',
    value: function clear() {
      (0, _utils.clearDefer)(this._id);
      delete this._id;
    }
  }]);

  return Task;
})();

exports['default'] = Task;
module.exports = exports['default'];