'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.wrapReplaceReducer = undefined;

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _toArray2 = require('babel-runtime/helpers/toArray');

var _toArray3 = _interopRequireDefault(_toArray2);

var _enhanceReducer = require('./enhanceReducer');

var _enhanceReducer2 = _interopRequireDefault(_enhanceReducer);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Wraps Store `replaceReducer` method. The implementation just calls
 * the original `replaceReducer` method but the provided next reducer
 * is enhanced.
 *
 * @param {Object} Original Store
 * @returns {Function} Wrapped `replaceReducer`
 */
var wrapReplaceReducer = exports.wrapReplaceReducer = function wrapReplaceReducer(store, deferEffects) {
  return function (nextReducer) {
    return store.replaceReducer((0, _enhanceReducer2.default)(nextReducer, deferEffects));
  };
};

/**
 * Creates enhanced store factory, which takes original `createStore` as argument.
 * Returns modified store which is capable of handling Reducers in form of Generators.
 *
 * @param {Function} Original createStore implementation to be enhanced
 * @returns {Function} Store factory
 */

exports.default = function (createStore) {
  return function (rootReducer, initialState) {
    var store = null;

    var deferEffects = function deferEffects(effects) {
      return setTimeout(function () {
        return effects.forEach(function (_ref) {
          var _ref2 = (0, _toArray3.default)(_ref);

          var fn = _ref2[0];

          var args = _ref2.slice(1);

          return fn.apply(undefined, [store.dispatch].concat((0, _toConsumableArray3.default)(args)));
        });
      }, 0);
    };

    store = createStore((0, _enhanceReducer2.default)(rootReducer, deferEffects), initialState);

    return (0, _extends3.default)({}, store, {
      replaceReducer: wrapReplaceReducer(store, deferEffects)
    });
  };
};